local NUM_TRADING_SLOTS = 7
local inventoryIndex = 0
local inventory = {}
local interfaceItem = {}
local interfaceIndex = 0
local color = "|cFF4FF4A2TradeFill:|r"

local MyAddon = LibStub("AceAddon-3.0"):NewAddon("MyAddon", "AceConsole-3.0", "AceEvent-3.0")

local classes = { "WARRIOR", "PALADIN", "HUNTER", "ROGUE", "PRIEST", "SHAMAN", "MAGE", "WARLOCK", "DRUID" }

local OptionFrame = CreateFrame("Frame", "MyIconButtonFrame", TradeFrame, "BackdropTemplate")
local ButtonA = CreateFrame("Button", "MyTradeButton", OptionFrame, "UIPanelButtonTemplate")
local ButtonB = CreateFrame("Button", "MyTradeButton", OptionFrame, "UIPanelButtonTemplate")
local ClearTrade = CreateFrame("Button", "MyTradeButton", OptionFrame, "UIPanelButtonTemplate")

local options = {
    name = "TradeFill",
    handler = MyAddon,
    type = "group",
    args = {
        menu = {
            type = "group",
            name = "",
            order = 0,
            args = {
                toggleBoxA = {
                    type = 'toggle',
                    name = "Show only full stacks",
                    desc = "Hide stacks that are not fill in dropdown menu",
                    get = function(info) return MyAddon.itemdb.profile.show end,
                    set = function(info, value) MyAddon.itemdb.profile.show = value end,
                    order = 0,
                },
                separatorA = {
                    type = "description",
                    name = " ",
                    order = 1,
                },
                iconA = {
                    type = "description",
                    fontSize = "medium",
                    name = function(info) return MyAddon:GetName("A") end,
                    image = function(info) return MyAddon:SetIconImage("A") end,
                    imageWidth = 32,
                    imageHeight = 32,
                    order = 2,
                },
                itemA = {
                    type = "select",
                    width = 1.5,
                    name = "Item to trade",
                    desc = "Select a item",
                    values = function() return MyAddon:SetOptionItem("A") end,
                    get = function() return MyAddon:GetInputItem("A") end,
                    set = function(info, value) MyAddon:SetInputItem("A", tonumber(value)) end,
                    order = 3,
                },
                separatorB = {
                    type = "description",
                    name = " ",
                    order = 4,
                },
                iconB = {
                    type = "description",
                    fontSize = "medium",
                    name = function(info) return MyAddon:GetName("B") end,
                    image = function(info) return MyAddon:SetIconImage("B") end,
                    imageWidth = 32,
                    imageHeight = 32,
                    order = 5,
                },
                itemB = {
                    type = "select",
                    width = 1.5,
                    name = "Item to trade",
                    desc = "Select a item",
                    values = function() return MyAddon:SetOptionItem("B") end,
                    get = function() return MyAddon:GetInputItem("B") end,
                    set = function(info, value) MyAddon:SetInputItem("B", tonumber(value)) end,
                    order = 6,
                },
                separatorC = {
                    type = "description",
                    name = " ",
                    order = 7,
                },
                toggleBoxB = {
                    type = 'toggle',
                    name = "Only full stacks",
                    desc = "Fill the trade window only with full stacks",
                    get = function(info) return MyAddon.itemdb.profile.stack end,
                    set = function(info, value) MyAddon.itemdb.profile.stack = value end,
                    order = 8,
                },
                toggleBoxC = {
                    type = 'toggle',
                    name = "Hide buttons",
                    desc = "Don't show extra buttons in trade window",
                    get = function(info) return MyAddon.itemdb.profile.button end,
                    set = function(info, value) MyAddon.itemdb.profile.button = value end,
                    order = 9,
                },
                separatorD = {
                    type = "description",
                    name = " ",
                    order = 10,
                },
                toggleBoxD = {
                    type = 'toggle',
                    name = "Fill trade when Solo",
                    desc = "Fill trade when you not in a group",
                    get = function() return MyAddon.itemdb.profile.solo end,
                    set = function(info, value) MyAddon.itemdb.profile.solo = value end,
                    order = 11,
                },
                toggleBoxE = {
                    type = 'toggle',
                    name = "Fill trade in Party",
                    desc = "Fill trade when your in a party group",
                    get = function() return MyAddon.itemdb.profile.party end,
                    set = function(info, value) MyAddon.itemdb.profile.party = value end,
                    order = 12,
                },
                toggleBoxF = {
                    type = 'toggle',
                    name = "Fill trade in Raid",
                    desc = "Fill trade when your in a raid group",
                    get = function() return MyAddon.itemdb.profile.raid end,
                    set = function(info, value) MyAddon.itemdb.profile.raid = value end,
                    order = 13,
                },
            },
            inline = true,
        },
        classes = {
            type = "group",
            name = "Class Settings Stacks",
            order = 14,
            args = {},
        },
    },
}

function capitalizeFirstLetter(str)
    return str:sub(1, 1):upper() .. str:sub(2):lower()
end

for _, class in pairs(classes) do
    local rPerc, gPerc, bPerc, argbHex = GetClassColor(class)
    options.args[class] = {
        type = "group",
        name = "|c".. argbHex .. capitalizeFirstLetter(class) .. "|r",
        args = {
            separatorA = {
                type = "description",
                name = "Number of stack to trade",
                fontSize = "medium",
                order = 0,
            },
            separatorB = {
                type = "description",
                name = " ",
                order = 1,
            },
            selectedOptionA = {
                type = "select",
                width = 1,
                name = function() return MyAddon:GetName("A") end,
                desc = "Choose an amount of stack you want to trade",
                values = function() return MyAddon:SetOptionStack("A") end,
                get = function() return tostring(MyAddon.stacksdb.profile[class].stackNumberA) end,
                set = function(info, value) MyAddon.stacksdb.profile[class].stackNumberA = tonumber(value) MyAddon:UpdateOptionStack(class, "A", tonumber(value)) end,
                order = 2,
            },
            separatorC = {
                type = "description",
                name = " ",
                order = 3,
            },
            selectedOptionB = {
                type = "select",
                width = 1,
                name = function() return MyAddon:GetName("B") end,
                desc = "Choose an amount of stack you want to trade",
                values = function() return MyAddon:SetOptionStack("B") end,
                get = function() return tostring(MyAddon.stacksdb.profile[class].stackNumberB) end,
                set = function(info, value) MyAddon.stacksdb.profile[class].stackNumberB = tonumber(value) MyAddon:UpdateOptionStack(class, "B", tonumber(value)) end,
                order = 4,
            },
        },
    }
end

function itemData(id)

    local itemName, itemLink, _, _, _, _, _, itemStackCount, _, iconTexture = GetItemInfo(id)

    return { ["name"] = itemName, ["link"] = itemLink, ["stack"] = itemStackCount, ["icon"] = iconTexture }
end

function MyAddon:enableDisableClassSetting(value, option)

    if option == "A" then

        if value == 0 then
            for _, class in pairs(classes) do
                options.args[class].args.selectedOptionA.disabled = true
            end
        else
            for _, class in pairs(classes) do
                options.args[class].args.selectedOptionA.disabled = false
            end
        end
    end

    if option == "B" then

        if value == 0 then
            for _, class in pairs(classes) do
                options.args[class].args.selectedOptionB.disabled = true
            end
        else
            for _, class in pairs(classes) do
                options.args[class].args.selectedOptionB.disabled = false
            end
        end
    end
end

function MyAddon:enableDisableButton(value, option)

    if option == "A" then

        if value == 0 or MyAddon.itemdb.profile.button == true then
            ButtonA:Hide()
        else
            ButtonA:Show()
        end
    end

    if option == "B" then

        if value == 0 or MyAddon.itemdb.profile.button == true then
            ButtonB:Hide()
        else
            ButtonB:Show()
        end
    end

    if MyAddon.itemdb.profile.A == 0 and MyAddon.itemdb.profile.B == 0 or MyAddon.itemdb.profile.button == true then
        ClearTrade:Hide()
    else
        ClearTrade:Show()
    end
end

function MyAddon:SetOptionStack(option)

    local selectOption = {}

    if option == "A" then
        for i = 0, NUM_TRADING_SLOTS do
            local itemId = MyAddon.itemdb.profile.itemIdA

            if itemId == 0 then
                selectOption[tostring(i)] = i
            else
                local itemName, itemLink, _, _, _, _, _, itemStackCount, _, iconTexture = GetItemInfo(itemId)

                selectOption[tostring(i)] = i .. " |T" .. GetItemIcon(itemId) .. ":0|t"
            end
        end
    end

    if option == "B" then
        for i = 0, NUM_TRADING_SLOTS do
            local itemId = MyAddon.itemdb.profile.itemIdB

            if itemId == 0 then
                selectOption[tostring(i)] = i
            else
                local itemName, itemLink, _, _, _, _, _, itemStackCount, _, iconTexture = GetItemInfo(itemId)

                selectOption[tostring(i)] = i .. " |T" .. GetItemIcon(itemId) .. ":0|t"
            end
        end
    end

    return selectOption
end

function MyAddon:SetOptionItem(option)

    for bag = 0, 4 do
        for slot = 1, C_Container.GetContainerNumSlots(bag) do
            local bagItem = C_Container.GetContainerItemInfo(bag, slot)

            if bagItem ~= nil then
                interfaceIndex = interfaceIndex + 1
                interfaceItem[interfaceIndex] = bagItem
            end
        end
    end

    local selectOption = {}

    selectOption["0"] = "None"

    for i, slot in pairs(interfaceItem) do
        local itemName, itemLink, _, _, _, _, _, itemStackCount, _, iconTexture = GetItemInfo(slot.itemID)

        if MyAddon.itemdb.profile.show == true then
            if slot.isBound == false and slot.stackCount == itemStackCount then
                selectOption[tostring(slot.itemID)] = "|T" .. GetItemIcon(slot.itemID) .. ":0|t " .. slot.hyperlink:gsub("[%[%]]", "")
            end
        else
            if slot.isBound == false then
                selectOption[tostring(slot.itemID)] = "|T" .. GetItemIcon(slot.itemID) .. ":0|t " .. slot.hyperlink:gsub("[%[%]]", "")
            end
        end

        if MyAddon.itemdb.profile.show == true  then

            if option == "A" and MyAddon.itemdb.profile.itemIdA ~= 0 then
                local item = MyAddon.itemdb.profile.itemIdA
                local _, itemLink, _, _, _, _, _, _, _, _ = GetItemInfo(item)

                selectOption[tostring(item)] = "|T" .. GetItemIcon(item) .. ":0|t " .. itemLink:gsub("[%[%]]", "")
            end

            if option == "B" and MyAddon.itemdb.profile.itemIdB ~= 0 then
                local item = MyAddon.itemdb.profile.itemIdB
                local _, itemLink, _, _, _, _, _, _, _, _ = GetItemInfo(item)

                selectOption[tostring(item)] = "|T" .. GetItemIcon(item) .. ":0|t " .. itemLink:gsub("[%[%]]", "")
            end
        end

    end
    interfaceItem = {}
    interfaceIndex = 0
    return selectOption
end

function MyAddon:UpdateOptionStack(class, option, value)

    local stackA = self.stacksdb.profile[class].stackNumberA
    local stackB = self.stacksdb.profile[class].stackNumberB

    local newValue = NUM_TRADING_SLOTS - value

    if option == "A" then
        if stackA + stackB > NUM_TRADING_SLOTS then

            self.stacksdb.profile[class].stackNumberB = newValue
        end
    end

    if option == "B" then
        if stackA + stackB > NUM_TRADING_SLOTS then

            self.stacksdb.profile[class].stackNumberA = newValue
        end
    end
end

function MyAddon:GetInputItem(option)

    if option == "A" then
        local value = MyAddon.itemdb.profile.itemIdA
        MyAddon:enableDisableClassSetting(value, option)
        MyAddon:enableDisableButton(value, option)
        return tostring(value)
    end

    if option == "B" then
        local value = MyAddon.itemdb.profile.itemIdB
        MyAddon:enableDisableClassSetting(value, option)
        MyAddon:enableDisableButton(value, option)
        return tostring(value)
    end
end

function MyAddon:SetInputItem(option, value)

    local item = itemData(value)

    if option == "A" then

        ButtonA:SetText(item.name)
        ButtonA:SetPoint("BOTTOM", OptionFrame, "BOTTOMRIGHT", 10 + ButtonA:GetFontString():GetStringWidth() / 2, -19)
        ButtonA:SetSize(ButtonA:GetFontString():GetStringWidth() + 20, 22)

        self.itemdb.profile.itemNameA = item.name
        self.itemdb.profile.itemIdA = value

        MyAddon:enableDisableClassSetting(value, option)
        MyAddon:enableDisableButton(value, option)
    end

    if option == "B" then

        ButtonB:SetText(item.name)
        ButtonB:SetPoint("BOTTOM", OptionFrame, "BOTTOMRIGHT", 10 + ButtonB:GetFontString():GetStringWidth() / 2, -38)
        ButtonB:SetSize(ButtonB:GetFontString():GetStringWidth() + 20, 22)

        self.itemdb.profile.itemNameB = item.name
        self.itemdb.profile.itemIdB = value

        MyAddon:enableDisableClassSetting(value, option)
        MyAddon:enableDisableButton(value, option)
    end
end

function MyAddon:GetName(option)

    if option == "A" then
        local item = itemData(self.itemdb.profile.itemIdA)

        if item.name then
            return item.link:gsub("[%[%]]", "")
        else
            return "Unknown"
        end
    end

    if option == "B" then
        local item = itemData(self.itemdb.profile.itemIdB)

        if item.name then
            return item.link:gsub("[%[%]]", "")
        else
            return "Unknown"
        end
    end
end

function MyAddon:SetIconImage(option)

    if option == "A" then
        local item = itemData(self.itemdb.profile.itemIdA)

        if item.name then
            return item.icon
        else
            return 134400
        end
    end

    if option == "B" then
        local item = itemData(self.itemdb.profile.itemIdB)

        if item.name then
            return item.icon
        else
            return 134400
        end
    end
end

LibStub("AceConfig-3.0"):RegisterOptionsTable("MyAddon", options)
local AceConfigDialog = LibStub("AceConfigDialog-3.0")
AceConfigDialog:AddToBlizOptions("MyAddon", "TradeFill")

local defaultsStacks = {
    profile = {
        ["**"] = { stackNumberA = "0", stackNumberB = "0" }
    }
}

local defaultsItems = {
    profile = {
        itemIdA = "0", itemIdB = "0", show = false, stack = false, button = false, solo = false, party = false, raid = false
    }
}

-----------------------------------------------
-----------------------------------------------

function groupStatus()
    local inRaid = IsInRaid()

    if inRaid then
        return "raid"
    elseif IsInGroup() then
        return "party"
    else
        return "solo"
    end
end

function tradePosition()
    for i = 1, MAX_TRADABLE_ITEMS do
        if GetTradePlayerItemLink(i) == nil then
            return i
        end
    end
end

function tradeFill(item, position)
    local _, itemLink = GetItemInfo(item)

    if next(inventory) == nil then
        print(color .. "Out of stock " .. itemLink .. " |T" .. GetItemIcon(item) .. ":0|t (item not found)")
    end

    if position == nil then
        print(color .. "Trade window is full!|r")
        return
    else
        for i = 1, inventoryIndex do
            local locked = C_Container.GetContainerItemInfo(inventory[i].bag, inventory[i].slot)
            local stackCount = itemData(inventory[i].item)

            if inventory[i].bound == true then
                print(color .. "Item is Soulbound " .. itemLink .. " |T" .. GetItemIcon(item) .. ":0|t (can't trade that)")
                return
            end

            if MyAddon.itemdb.profile.stack == false then
                if inventory[i].item == item and locked.isLocked == false then
                    ClearCursor()
                    C_Container.PickupContainerItem(inventory[i].bag, inventory[i].slot)
                    ClickTradeButton(position)
                    break
                end

                if i >= inventoryIndex then
                    print(color .. "Out of stock " .. itemLink .. " |T" .. GetItemIcon(item) .. ":0|t (any stacks)")
                    return
                end
            end

            if MyAddon.itemdb.profile.stack == true then
                if inventory[i].item == item and locked.isLocked == false and inventory[i].count == stackCount.stack then
                    ClearCursor()
                    C_Container.PickupContainerItem(inventory[i].bag, inventory[i].slot)
                    ClickTradeButton(position)
                    break
                end
                if i >= inventoryIndex then
                    print(color .. "Out of stock " .. itemLink .. " |T" .. GetItemIcon(item) .. ":0|t (full stack)")
                    return
                end
            end
        end
    end
end

function start()
    local itemA = MyAddon.itemdb.profile.itemIdA
    local itemB = MyAddon.itemdb.profile.itemIdB

    local _, ClassName, _ = UnitClass("npc")

    for bag = 0, 4 do
        for slot = 1, C_Container.GetContainerNumSlots(bag) do
            local bagItem = C_Container.GetContainerItemInfo(bag, slot)

            if bagItem ~= nil then
                if bagItem.itemID == itemA or bagItem.itemID == itemB then
                    inventoryIndex = inventoryIndex + 1
                    inventory[inventoryIndex] = {
                        ["bag"] = bag,
                        ["slot"] = slot,
                        ["item"] = bagItem.itemID,
                        ["bound"] = bagItem.isBound,
                        ["count"] = bagItem.stackCount,
                        ["link"] = bagItem.hyperlink
                    }
                end
            end
        end
    end

    if (MyAddon.itemdb.profile.solo == true and groupStatus() == "solo") or (MyAddon.itemdb.profile.party == true and groupStatus() == "party") or (MyAddon.itemdb.profile.raid == true and groupStatus() == "raid") then

        if itemA > 0 then
            for i = 1, MyAddon.stacksdb.profile[ClassName].stackNumberA do
                tradeFill(itemA, tradePosition())
            end
        end

        if itemB > 0 then
            for i = 1, MyAddon.stacksdb.profile[ClassName].stackNumberB do
                tradeFill(itemB, tradePosition())
            end
        end
    end
end

function close()
    inventoryIndex = 0
    inventory = {}
end

function MyAddon:OnInitialize()
    self.stacksdb = LibStub("AceDB-3.0"):New("StacksDB", defaultsStacks)
    self.itemdb = LibStub("AceDB-3.0"):New("ItemDB", defaultsItems)

    local faction = UnitFactionGroup("player")

    if faction == "Alliance" then
        options.args["SHAMAN"].hidden = true
    elseif faction == "Horde" then
        options.args["PALADIN"].hidden = true
    end

    OptionFrame:SetWidth(200)
    OptionFrame:SetHeight(600)
    OptionFrame:SetPoint("BOTTOMRIGHT", TradeFrame, "BOTTOMRIGHT", 0, 0)

    ButtonA:SetText(MyAddon.itemdb.profile.itemNameA)
    ButtonA:SetPoint("BOTTOM", OptionFrame, "BOTTOMRIGHT", 10 + ButtonA:GetFontString():GetStringWidth() / 2, -19)
    ButtonA:SetSize(ButtonA:GetFontString():GetStringWidth() + 20, 22)
    local function ButtonItemA()
        tradeFill(MyAddon.itemdb.profile.itemIdA, tradePosition())
    end
    ButtonA:SetScript("OnClick", ButtonItemA)
    MyAddon:enableDisableButton(MyAddon.itemdb.profile.itemIdA, "A")

    ButtonB:SetText(MyAddon.itemdb.profile.itemNameB)
    ButtonB:SetPoint("BOTTOM", OptionFrame, "BOTTOMRIGHT", 10 + ButtonB:GetFontString():GetStringWidth() / 2, -38)
    ButtonB:SetSize(ButtonB:GetFontString():GetStringWidth() + 20, 22)
    local function ButtonItemB()
        tradeFill(MyAddon.itemdb.profile.itemIdB, tradePosition())
    end
    ButtonB:SetScript("OnClick", ButtonItemB)
    MyAddon:enableDisableButton(MyAddon.itemdb.profile.itemIdB, "B")

    ClearTrade:SetText("Clear")
    ClearTrade:SetPoint("BOTTOMRIGHT", OptionFrame, "BOTTOMRIGHT", 73, 5)
    ClearTrade:SetSize(75, 22)
    local function ButtonClear()
        for i = 1, MAX_TRADABLE_ITEMS do
            ClearCursor()
            ClickTradeButton(i)
        end
        ClearCursor()
    end
    ClearTrade:SetScript("OnClick", ButtonClear)
end

MyAddon:RegisterEvent("TRADE_SHOW", start)
MyAddon:RegisterEvent("TRADE_CLOSED", close)